@@ -454,13 +454,34 @@ class Expression(object):
         from pymbolic.mapper.stringifier import PREC_NONE
         return self.stringifier()()(self, PREC_NONE)
 
+    def _safe_repr(self, limit=10):
+        if limit <= 0:
+            return "..."
+
+        def strify_child(child, limit):
+            if isinstance(child, tuple):
+                # Make sure limit propagates at least through tuples
+
+                return "(%s%s)" % (
+                        ", ".join(strify_child(i, limit-1) for i in child),
+                        "," if len(child) == 1 else "")
+
+            elif isinstance(child, Expression):
+                return child._safe_repr(limit=limit-1)
+            else:
+                return repr(child)
+
+        initargs_str = ", ".join(
+                strify_child(i, limit-1)
+                for i in self.__getinitargs__())
+
+        return "%s(%s)" % (self.__class__.__name__, initargs_str)
+
     def __repr__(self):
         """Provides a default :func:`repr` based on
         the Python pickling interface :meth:`__getinitargs__`.
         """
-        initargs_str = ", ".join(repr(i) for i in self.__getinitargs__())
-
-        return "%s(%s)" % (self.__class__.__name__, initargs_str)
+        return self._safe_repr()
 
     # }}}
 
@@ -520,7 +541,7 @@ class Expression(object):
     # {{{ logical op constructors
 
     def not_(self):
-        """Return *self* wrapped in a :class:`LogicalNot.
+        """Return *self* wrapped in a :class:`LogicalNot`.
 
         .. versionadded:: 2015.2
         """
@@ -607,7 +628,7 @@ class Expression(object):
     # }}}
 
     def __iter__(self):
-        # prevent infinite loops (e.g. when inseserting into numpy arrays)
+        # prevent infinite loops (e.g. when inserting into numpy arrays)
         raise TypeError("expression types are not iterable")
 
 
@@ -682,7 +703,7 @@ class Call(AlgebraicLeaf):
 
     .. attribute:: parameters
 
-        A :class:`tuple` of positional paramters, each element
+        A :class:`tuple` of positional parameters, each element
         of which is a :class:`Expression` or a constant.
 
     """
@@ -718,7 +739,7 @@ class CallWithKwargs(AlgebraicLeaf):
 
     .. attribute:: parameters
 
-        A :class:`tuple` of positional paramters, each element
+        A :class:`tuple` of positional parameters, each element
         of which is a :class:`Expression` or a constant.
 
     .. attribute:: kw_parameters
@@ -757,6 +778,19 @@ class CallWithKwargs(AlgebraicLeaf):
                     list(self.kw_parameters.items()),
                     key=lambda item: item[0])))
 
+    def __setstate__(self, state):
+        # CallWithKwargs must override __setstate__ because during pickling the
+        # kw_parameters are converted to tuple, which needs to be converted
+        # back to dict.
+        assert len(self.init_arg_names) == len(state)
+        function, parameters, kw_parameters = state
+
+        self.function = function
+        self.parameters = parameters
+        if not isinstance(kw_parameters, dict):
+            kw_parameters = dict(kw_parameters)
+        self.kw_parameters = kw_parameters
+
     mapper_method = intern("map_call_with_kwargs")
 
 
